@@ -1,10 +1,27 @@
 import urllib.request
 import json
-import time
+from datetime import datetime
 
 from i3pystatus import IntervalModule
 from i3pystatus.core.util import internet, require, user_open
 
+import locale
+import threading
+from contextlib import contextmanager
+
+LOCALE_LOCK = threading.Lock()
+
+
+@contextmanager
+def setlocale(name):
+    # To deal with locales only in this module and keep it thread save
+    with LOCALE_LOCK:
+        saved = locale.setlocale(locale.LC_ALL)
+        try:
+            yield locale.setlocale(locale.LC_ALL, name)
+        finally:
+            locale.setlocale(locale.LC_ALL, saved)
+
 
 class Bitcoin(IntervalModule):
 
@@ -12,8 +29,10 @@ class Bitcoin(IntervalModule):
     This module fetches and displays current Bitcoin market prices and
     optionally monitors transactions to and from a list of user-specified
     wallet addresses. Market data is pulled from the BitcoinAverage Price
-    Index API <https://bitcoinaverage.com> while transaction data is pulled
-    from blockchain.info <https://blockchain.info/api/blockchain_api>.
+    Index API <https://bitcoinaverage.com> and it is possible to specify
+    the exchange to be monitored.
+    Transaction data is pulled from blockchain.info
+    <https://blockchain.info/api/blockchain_api>.
 
     .. rubric:: Available formatters
 
@@ -22,12 +41,16 @@ class Bitcoin(IntervalModule):
     * {bid_price}
     * {daily_average}
     * {volume}
+    * {volume_thousend}
+    * {volume_percent}
+    * {age}
     * {status}
     * {last_tx_type}
     * {last_tx_addr}
     * {last_tx_value}
     * {balance_btc}
     * {balance_fiat}
+    * {symbol}
 
     """
 
@@ -36,35 +59,63 @@ class Bitcoin(IntervalModule):
         ("currency", "Base fiat currency used for pricing."),
         ("wallet_addresses", "List of wallet address(es) to monitor."),
         ("color", "Standard color"),
+        ("exchange", "Get ticker from a custom exchange instead"),
         ("colorize", "Enable color change on price increase/decrease"),
         ("color_up", "Color for price increases"),
         ("color_down", "Color for price decreases"),
-        ("leftclick", "URL to visit or command to run on left click"),
-        ("rightclick", "URL to visit or command to run on right click"),
         ("interval", "Update interval."),
+        ("symbol", "Symbol for bitcoin sign"),
         "status"
     )
-    format = "฿ {status}{last_price}"
+    format = "{symbol} {status}{last_price}"
     currency = "USD"
+    exchange = None
+    symbol = "฿"
     wallet_addresses = ""
     color = "#FFFFFF"
     colorize = False
     color_up = "#00FF00"
     color_down = "#FF0000"
-    leftclick = "electrum"
-    rightclick = "https://bitcoinaverage.com/"
     interval = 600
     status = {
         "price_up": "▲",
         "price_down": "▼",
     }
 
+    on_leftclick = "electrum"
+    on_rightclick = ["open_something", "https://bitcoinaverage.com/"]
+
     _price_prev = 0
 
+    def _get_age(self, bitcoinaverage_timestamp):
+        with setlocale('C'):  # Deal with locales (months name differ)
+            # Assume format is always utc, to avoid import pytz
+            utc_tstamp = datetime.strptime(
+                bitcoinaverage_timestamp.split(', ')[1],
+                u'%d %b %Y %H:%M:%S -0000')
+        diff = datetime.utcnow() - utc_tstamp
+        return int(diff.total_seconds())
+
+    def _query_api(self, api_url):
+        url = "{}{}".format(api_url, self.currency.upper())
+        response = urllib.request.urlopen(url).read().decode("utf-8")
+        return json.loads(response)
+
     def _fetch_price_data(self):
-        api = "https://api.bitcoinaverage.com/ticker/global/"
-        url = "{}{}".format(api, self.currency.upper())
-        return json.loads(urllib.request.urlopen(url).read().decode("utf-8"))
+        if self.exchange is None:
+            api_url = "https://api.bitcoinaverage.com/ticker/global/"
+            return self._query_api(api_url)
+        else:
+            api_url = "https://api.bitcoinaverage.com/exchanges/"
+            ret = self._query_api(api_url)
+            exchange = ret[self.exchange.lower()]
+            # Adapt values to global ticker format
+            exchange['ask'] = exchange['rates']['ask']
+            exchange['bid'] = exchange['rates']['bid']
+            exchange['last'] = exchange['rates']['last']
+            exchange['24h_avg'] = None
+            exchange['timestamp'] = ret['timestamp']
+            return exchange
 
     def _fetch_blockchain_data(self):
         api = "https://blockchain.info/multiaddr?active="
@@ -75,12 +126,17 @@ class Bitcoin(IntervalModule):
     @require(internet)
     def run(self):
         price_data = self._fetch_price_data()
+
         fdict = {
+            "symbol": self.symbol,
             "daily_average": price_data["24h_avg"],
             "ask_price": price_data["ask"],
             "bid_price": price_data["bid"],
             "last_price": price_data["last"],
             "volume": price_data["volume_btc"],
+            "volume_thousend": price_data["volume_btc"] / 1000,
+            "volume_percent": price_data["volume_percent"],
+            "age": self._get_age(price_data['timestamp'])
         }
 
         if self._price_prev and fdict["last_price"] > self._price_prev:
@@ -117,13 +173,14 @@ class Bitcoin(IntervalModule):
                 else:
                     fdict["last_tx_type"] = "sent"
 
+        self.data = fdict
         self.output = {
             "full_text": self.format.format(**fdict),
             "color": color,
         }
 
-    def on_leftclick(self):
-        user_open(self.leftclick)
-
-    def on_rightclick(self):
-        user_open(self.rightclick)
+    def open_something(self, url_or_command):
+        """
+        Wrapper function, to pass the arguments to user_open
+        """
+        user_open(url_or_command)
