@@ -67,17 +67,32 @@ class ExceptionWrapper(Wrapper):
         try:
             self.workload()
         except:
-            sys.stderr.write("Exception in {thread} at {time}\n".format(
+            message = "Exception in {thread} at {time}, module {name}".format(
                 thread=threading.current_thread().name,
-                time=time.strftime("%c")
-            ))
-            traceback.print_exc(file=sys.stderr)
-            sys.stderr.flush()
-            if hasattr(self.workload, "output"):
-                self.workload.output = {
-                    "full_text": "{}: {}".format(self.workload.__class__.__name__, sys.exc_info()[1]),
-                    "color": "#FF0000",
-                }
+                time=time.strftime("%c"),
+                name=self.workload.__class__.__name__
+            )
+            if hasattr(self.workload, "logger"):
+                self.workload.logger.error(message, exc_info=True)
+            self.workload.output = {
+                "full_text": self.format_exception(),
+                "color": "#FF0000",
+            }
+
+    def format_exception(self):
+        type, value, _ = sys.exc_info()
+        exception = self.truncate_error("%s: %s" % (type.__name__, value))
+        return "%s: %s" % (self.workload.__class__.__name__, exception)
+
+    def truncate_error(self, exception_message):
+        if hasattr(self.workload, 'max_error_len'):
+            error_len = self.workload.max_error_len
+            if len(exception_message) > error_len:
+                return exception_message[:error_len] + '…'
+            else:
+                return exception_message
+        else:
+            return exception_message
 
 
 class WorkloadWrapper(Wrapper):
