@@ -3,6 +3,8 @@ import functools
 import re
 import socket
 import string
+import inspect
+from threading import Timer, RLock
 
 
 def lchop(string, prefix):
@@ -335,7 +337,7 @@ def require(predicate):
 
     .. seealso::
 
-        :py:func:`internet`
+        :py:class:`internet`
 
     """
 
@@ -351,43 +353,90 @@ def require(predicate):
     return decorator
 
 
-def internet():
+class internet:
     """
-    Checks for a internet connection by connecting to a Google DNS
-    server.
+    Checks for internet connection by connecting to a server.
+
+    Used server is determined by the `address` class variable which consists of
+    server host name and port number.
+
+    :rtype: bool
+
+    .. seealso::
+
+        :py:func:`require`
 
-    :returns: True if internet connection is available
     """
-    try:
-        socket.create_connection(("google-public-dns-a.google.com", 53), 1).close()
-        return True
-    except OSError:
-        return False
+    address = ("google-public-dns-a.google.com", 53)
+
+    def __new__(cls):
+        try:
+            socket.create_connection(cls.address, 1).close()
+            return True
+        except OSError:
+            return False
 
 
-def make_graph(values, upper_limit=100.0):
+def make_graph(values, lower_limit=0.0, upper_limit=100.0, style="blocks"):
     """
     Draws a graph made of unicode characters.
 
     :param values: An array of values to graph.
-    :param upper_limit: Maximum value for the y axis.
+    :param lower_limit: Minimum value for the y axis (or None for dynamic).
+    :param upper_limit: Maximum value for the y axis (or None for dynamic).
+    :param style: Drawing style ('blocks', 'braille-fill', 'braille-peak', or 'braille-snake').
     :returns: Bar as a string
     """
+
     values = [float(n) for n in values]
-
-    # Add the upper limit to the end of the array so the graph doesn't distort
-    # as high values drop off the end.
-    values.append(float(upper_limit))
-
-    bar = u'_▁▂▃▄▅▆▇█'
-    bar_count = len(bar) - 1
     mn, mx = min(values), max(values)
+    mn = mn if lower_limit is None else min(mn, float(lower_limit))
+    mx = mx if upper_limit is None else max(mx, float(upper_limit))
     extent = mx - mn
-    if extent == 0:
-        graph = '_' * len(values)
+
+    if style == 'blocks':
+        bar = '_▁▂▃▄▅▆▇█'
+        bar_count = len(bar) - 1
+        if extent == 0:
+            graph = '_' * len(values)
+        else:
+            graph = ''.join(bar[int((n - mn) / extent * bar_count)] for n in values)
+    elif style in ['braille-fill', 'braille-peak', 'braille-snake']:
+        # idea from https://github.com/asciimoo/drawille
+        # unicode values from http://en.wikipedia.org/wiki/Braille
+
+        vpad = values if len(values) % 2 == 0 else values + [mn]
+        vscale = [round(4 * (vp - mn) / extent) for vp in vpad]
+        l = len(vscale) // 2
+
+        # do the 2-character collapse separately for clarity
+        if 'fill' in style:
+            vbits = [[0, 0x40, 0x44, 0x46, 0x47][vs] for vs in vscale]
+        elif 'peak' in style:
+            vbits = [[0, 0x40, 0x04, 0x02, 0x01][vs] for vs in vscale]
+        else:
+            assert('snake' in style)
+            # there are a few choices for what to put last in vb2.
+            # arguable vscale[-1] from the _previous_ call is best.
+            vb2 = [vscale[0]] + vscale + [0]
+            vbits = []
+            for i in range(1, l + 1):
+                c = 0
+                for j in range(min(vb2[i - 1], vb2[i], vb2[i + 1]), vb2[i] + 1):
+                    c |= [0, 0x40, 0x04, 0x02, 0x01][j]
+                vbits.append(c)
+
+        # 2-character collapse
+        graph = ''
+        for i in range(0, l, 2):
+            b1 = vbits[i]
+            b2 = vbits[i + 1]
+            if b2 & 0x40:
+                b2 = b2 - 0x30
+            b2 = b2 << 3
+            graph += chr(0x2800 + b1 + b2)
     else:
-        graph = ''.join(bar[int((n - mn) / extent * bar_count)]
-                        for n in values[:len(values) - 1])  # Don't show the upper limit value.
+        raise NotImplementedError("Graph drawing style '%s' unimplemented." % style)
     return graph
 
 
@@ -399,7 +448,7 @@ def make_vertical_bar(percentage, width=1):
     :param width: How many characters wide the bar should be.
     :returns: Bar as a String
     """
-    bar = u' _▁▂▃▄▅▆▇█'
+    bar = ' _▁▂▃▄▅▆▇█'
     percentage //= 10
     if percentage < 0:
         output = bar[0]
@@ -431,7 +480,7 @@ def make_bar(percentage):
 def user_open(url_or_command):
     """Open the specified paramater in the web browser if a URL is detected,
     othewrise pass the paramater to the shell as a subprocess. This function
-    is inteded to bu used in on_leftclick()/on_rightclick() events.
+    is inteded to bu used in on_leftclick/on_rightclick callbacks.
 
     :param url_or_command: String containing URL or command
     """
@@ -439,7 +488,103 @@ def user_open(url_or_command):
     scheme = urlparse(url_or_command).scheme
     if scheme == 'http' or scheme == 'https':
         import webbrowser
-        webbrowser.open(url_or_command)
+        import os
+        # webbrowser.open() sometimes prints a message for some reason and confuses i3
+        # Redirect stdout briefly to prevent this from happening.
+        savout = os.dup(1)
+        os.close(1)
+        os.open(os.devnull, os.O_RDWR)
+        try:
+            webbrowser.open(url_or_command)
+        finally:
+            os.dup2(savout, 1)
     else:
         import subprocess
         subprocess.Popen(url_or_command, shell=True)
+
+
+class MultiClickHandler(object):
+    def __init__(self, callback_handler, timeout):
+        self.callback_handler = callback_handler
+        self.timeout = timeout
+
+        self.lock = RLock()
+
+        self._timer_id = 0
+        self.timer = None
+        self.button = None
+        self.cb = None
+        self.kwargs = None
+
+    def set_timer(self, button, cb, **kwargs):
+        with self.lock:
+            self.clear_timer()
+
+            self.timer = Timer(self.timeout,
+                               self._timer_function,
+                               args=[self._timer_id])
+            self.button = button
+            self.cb = cb
+            self.kwargs = kwargs
+
+            self.timer.start()
+
+    def clear_timer(self):
+        with self.lock:
+            if self.timer is None:
+                return
+
+            self._timer_id += 1  # Invalidate existent timer
+
+            self.timer.cancel()  # Cancel the existent timer
+
+            self.timer = None
+            self.button = None
+            self.cb = None
+
+    def _timer_function(self, timer_id):
+        with self.lock:
+            if self._timer_id != timer_id:
+                return
+            self.callback_handler(self.button, self.cb, **self.kwargs)
+            self.clear_timer()
+
+    def check_double(self, button):
+        if self.timer is None:
+            return False
+
+        ret = True
+        if button != self.button:
+            self.callback_handler(self.button, self.cb, **self.kwargs)
+            ret = False
+
+        self.clear_timer()
+        return ret
+
+
+def get_module(function):
+    """Function decorator for retrieving the ``self`` argument from the stack.
+
+    Intended for use with callbacks that need access to a modules variables, for example:
+
+    .. code:: python
+
+        from i3pystatus import Status, get_module
+        from i3pystatus.core.command import execute
+        status = Status(...)
+        # other modules etc.
+        @get_module
+        def display_ip_verbose(module):
+            execute('sh -c "ip addr show dev {dev} | xmessage -file -"'.format(dev=module.interface))
+        status.register("network", interface="wlan1", on_leftclick=display_ip_verbose)
+    """
+    @functools.wraps(function)
+    def call_wrapper(*args, **kwargs):
+        stack = inspect.stack()
+        caller_frame_info = stack[1]
+        self = caller_frame_info[0].f_locals["self"]
+        # not completly sure whether this is necessary
+        # see note in Python docs about stack frames
+        del stack
+        function(self, *args, **kwargs)
+    return call_wrapper
