@@ -150,9 +150,10 @@ class ParcelTracker(IntervalModule):
         "format",
         "name",
     )
-    required = ("instance",)
+    required = ("instance", "name")
 
     format = "{name}:{progress}"
+    on_leftclick = "open_browser"
 
     @require(internet)
     def run(self):
@@ -161,10 +162,11 @@ class ParcelTracker(IntervalModule):
         }
         fdict.update(self.instance.status())
 
+        self.data = fdict
         self.output = {
             "full_text": self.format.format(**fdict).strip(),
             "instance": self.name,
         }
 
-    def on_leftclick(self):
+    def open_browser(self):
         webbrowser.open_new_tab(self.instance.get_url())
