@@ -1,12 +1,11 @@
-import os
-import shutil
+import re
+import subprocess
+
+from i3pystatus import Module
 from i3pystatus.core.color import ColorRangeModule
 from i3pystatus.core.util import make_vertical_bar, make_bar
 from .pulse import *
 
-from i3pystatus import Module
-import subprocess
-
 
 class PulseAudio(Module, ColorRangeModule):
     """
@@ -15,7 +14,7 @@ class PulseAudio(Module, ColorRangeModule):
     - Requires amixer for toggling mute and incrementing/decrementing volume on scroll.
     - Depends on the PyPI colour module - https://pypi.python.org/pypi/colour/0.0.5
 
-    .. rubric:: Available formatters:
+    .. rubric:: Available formatters
 
     * `{volume}` — volume in percent (0...100)
     * `{db}` — volume in decibels relative to 100 %, i.e. 100 % = 0 dB, 50 % = -18 dB, 0 % = -infinity dB
@@ -50,6 +49,12 @@ class PulseAudio(Module, ColorRangeModule):
     bar_type = 'vertical'
     vertical_bar_width = 2
 
+    on_rightclick = "switch_mute"
+    on_doubleleftclick = "change_sink"
+    on_leftclick = "pavucontrol"
+    on_upscroll = "increase_volume"
+    on_downscroll = "decrease_volume"
+
     def init(self):
         """Creates context, when context is ready context_notify_cb is called"""
         # Wrap callback methods in appropriate ctypefunc instances so
@@ -73,24 +78,35 @@ class PulseAudio(Module, ColorRangeModule):
         pa_threaded_mainloop_start(_mainloop)
 
         self.colors = self.get_hex_color_range(self.color_muted, self.color_unmuted, 100)
-
-        # Check that we have amixer for toggling mute/unmute and incrementing/decrementing volume
-        self.has_amixer = shutil.which('alsamixer') is not None
+        self.sinks = []
 
     def request_update(self, context):
         """Requests a sink info update (sink_info_cb is called)"""
         pa_operation_unref(pa_context_get_sink_info_by_name(
-            context, self.sink, self._sink_info_cb, None))
+            context, self.sink.encode(), self._sink_info_cb, None))
 
     def success_cb(self, context, success, userdata):
         pass
 
+    @property
+    def sink(self):
+        self.sinks = subprocess.check_output(['pactl', 'list', 'short', 'sinks'],
+                                             universal_newlines=True).splitlines()
+        bestsink = None
+        state = 'DEFAULT'
+        for sink in self.sinks:
+            attribs = sink.split()
+            sink_state = attribs[-1]
+            if sink_state == 'RUNNING':
+                bestsink = attribs[1]
+                state = 'RUNNING'
+            elif sink_state in ('IDLE', 'SUSPENDED') and state == 'DEFAULT':
+                bestsink = attribs[1]
+        return bestsink
+
     def server_info_cb(self, context, server_info_p, userdata):
         """Retrieves the default sink and calls request_update"""
         server_info = server_info_p.contents
-
-        self.sink = server_info.default_sink_name
-
         self.request_update(context)
 
     def context_notify_cb(self, context, _):
@@ -108,10 +124,15 @@ class PulseAudio(Module, ColorRangeModule):
             pa_context_set_subscribe_callback(context, self._update_cb, None)
 
             pa_operation_unref(pa_context_subscribe(
-                context, PA_SUBSCRIPTION_EVENT_CHANGE | PA_SUBSCRIPTION_MASK_SINK, self._success_cb, None))
+                context, PA_SUBSCRIPTION_EVENT_CHANGE | PA_SUBSCRIPTION_MASK_SINK | PA_SUBSCRIPTION_MASK_SERVER, self._success_cb, None))
 
     def update_cb(self, context, t, idx, userdata):
         """A sink property changed, calls request_update"""
+
+        if t & PA_SUBSCRIPTION_EVENT_FACILITY_MASK == PA_SUBSCRIPTION_EVENT_SERVER:
+            pa_operation_unref(
+                pa_context_get_server_info(context, self._server_info_cb, None))
+
         self.request_update(context)
 
     def sink_info_cb(self, context, sink_info_p, _, __):
@@ -155,24 +176,25 @@ class PulseAudio(Module, ColorRangeModule):
                     volume_bar=volume_bar),
             }
 
-    def on_leftclick(self):
-        subprocess.Popen(["pavucontrol"])
+            self.send_output()
 
-    def on_rightclick(self):
-        if self.has_amixer:
-            command = "amixer -q -D pulse sset Master "
-            if self.currently_muted:
-                command += 'unmute'
-            else:
-                command += 'mute'
-            subprocess.Popen(command.split())
+    def change_sink(self):
+        curr_sink = self.sink
+        sinks = list(s.split()[1] for s in self.sinks)
+        next_sink = (sinks.index(curr_sink) + 1) % len(sinks)
 
-    def on_upscroll(self):
-        if self.has_amixer:
-            command = "amixer -q -D pulse sset Master %s%%+" % self.step
-            subprocess.Popen(command.split())
+        sink_inputs = subprocess.check_output("pacmd list-sink-inputs".split(),
+                                              universal_newlines=True)
+        for input_index in re.findall('index:\s+(\d+)', sink_inputs):
+            command = "pacmd move-sink-input {} {}".format(input_index, sinks[next_sink])
+            subprocess.call(command.split())
+        subprocess.call("pacmd set-default-sink {}".format(sinks[next_sink]).split())
 
-    def on_downscroll(self):
-        if self.has_amixer:
-            command = "amixer -q -D pulse sset Master %s%%-" % self.step
-            subprocess.Popen(command.split())
+    def switch_mute(self):
+        subprocess.call(['pactl', 'set-sink-mute', self.sink, "toggle"])
+
+    def increase_volume(self):
+        subprocess.call(['pactl', 'set-sink-volume', self.sink, "+%s%%" % self.step])
+
+    def decrease_volume(self):
+        subprocess.call(['pactl', 'set-sink-volume', self.sink, "-%s%%" % self.step])
